@@ -2,7 +2,7 @@
  *                                                                         *
  *   libgig - C++ cross-platform Gigasampler format file access library    *
  *                                                                         *
- *   Copyright (C) 2003-2007 by Christian Schoenebeck                      *
+ *   Copyright (C) 2003-2009 by Christian Schoenebeck                      *
  *                              <cuse@users.sourceforge.net>               *
  *                                                                         *
  *   This library is free software; you can redistribute it and/or modify  *
@@ -21,6 +21,8 @@
  *   MA  02111-1307  USA                                                   *
  ***************************************************************************/
 
+#include <algorithm>
+#include <set>
 #include <string.h>
 
 #include "RIFF.h"
@@ -58,6 +60,8 @@ namespace RIFF {
         ulPos      = 0;
         pParent    = NULL;
         pChunkData = NULL;
+        CurrentChunkSize = 0;
+        NewChunkSize = 0;
         ulChunkDataSize = 0;
         ChunkID    = CHUNK_ID_RIFF;
         this->pFile = pFile;
@@ -72,6 +76,8 @@ namespace RIFF {
         pParent       = Parent;
         ulPos         = 0;
         pChunkData    = NULL;
+        CurrentChunkSize = 0;
+        NewChunkSize = 0;
         ulChunkDataSize = 0;
         ReadHeader(StartPos);
     }
@@ -82,14 +88,14 @@ namespace RIFF {
         this->pParent    = pParent;
         ulPos            = 0;
         pChunkData       = NULL;
-        ulChunkDataSize  = 0;
         ChunkID          = uiChunkID;
+        ulChunkDataSize  = 0;
         CurrentChunkSize = 0;
         NewChunkSize     = uiBodySize;
     }
 
     Chunk::~Chunk() {
-        if (CurrentChunkSize != NewChunkSize) pFile->UnlogResized(this);
+        if (pFile) pFile->UnlogResized(this);
         if (pChunkData) delete[] pChunkData;
     }
 
@@ -97,6 +103,8 @@ namespace RIFF {
         #if DEBUG
         std::cout << "Chunk::Readheader(" << fPos << ") ";
         #endif // DEBUG
+        ChunkID = 0;
+        NewChunkSize = CurrentChunkSize = 0;
         #if POSIX
         if (lseek(pFile->hFileRead, fPos, SEEK_SET) != -1) {
             read(pFile->hFileRead, &ChunkID, 4);
@@ -127,8 +135,8 @@ namespace RIFF {
             }
             #if DEBUG
             std::cout << "ckID=" << convertToString(ChunkID) << " ";
-            std::cout << "ckSize=" << ChunkSize << " ";
-            std::cout << "bEndianNative=" << bEndianNative << std::endl;
+            std::cout << "ckSize=" << CurrentChunkSize << " ";
+            std::cout << "bEndianNative=" << pFile->bEndianNative << std::endl;
             #endif // DEBUG
             NewChunkSize = CurrentChunkSize;
         }
@@ -224,7 +232,7 @@ namespace RIFF {
        #if DEBUG
        std::cout << "Chunk::Remainingbytes()=" << CurrentChunkSize - ulPos << std::endl;
        #endif // DEBUG
-        return CurrentChunkSize - ulPos;
+        return (CurrentChunkSize > ulPos) ? CurrentChunkSize - ulPos : 0;
     }
 
     /**
@@ -273,6 +281,7 @@ namespace RIFF {
        #if DEBUG
        std::cout << "Chunk::Read(void*,ulong,ulong)" << std::endl;
        #endif // DEBUG
+        if (ulStartPos == 0) return 0; // is only 0 if this is a new chunk, so nothing to read (yet)
         if (ulPos >= CurrentChunkSize) return 0;
         if (ulPos + WordCount * WordSize >= CurrentChunkSize) WordCount = (CurrentChunkSize - ulPos) / WordSize;
         #if POSIX
@@ -725,7 +734,7 @@ namespace RIFF {
      * @see ReleaseChunkData()
      */
     void* Chunk::LoadChunkData() {
-        if (!pChunkData && pFile->Filename != "") {
+        if (!pChunkData && pFile->Filename != "" && ulStartPos != 0) {
             #if POSIX
             if (lseek(pFile->hFileRead, ulStartPos, SEEK_SET) == -1) return NULL;
             #elif defined(WIN32)
@@ -942,6 +951,10 @@ namespace RIFF {
       #if DEBUG
       std::cout << "List::~List()" << std::endl;
       #endif // DEBUG
+        DeleteChunkList();
+    }
+
+    void List::DeleteChunkList() {
         if (pSubChunks) {
             ChunkList::iterator iter = pSubChunks->begin();
             ChunkList::iterator end  = pSubChunks->end();
@@ -950,8 +963,12 @@ namespace RIFF {
                 iter++;
             }
             delete pSubChunks;
+            pSubChunks = NULL;
+        }
+        if (pSubChunksMap) {
+            delete pSubChunksMap;
+            pSubChunksMap = NULL;
         }
-        if (pSubChunksMap) delete pSubChunksMap;
     }
 
     /**
@@ -1152,6 +1169,8 @@ namespace RIFF {
         pSubChunks->push_back(pNewChunk);
         (*pSubChunksMap)[uiChunkID] = pNewChunk;
         pNewChunk->Resize(uiBodySize);
+        NewChunkSize += CHUNK_HEADER_SIZE;
+        pFile->LogAsResized(this);
         return pNewChunk;
     }
 
@@ -1187,6 +1206,8 @@ namespace RIFF {
         List* pNewListChunk = new List(pFile, this, uiListType);
         pSubChunks->push_back(pNewListChunk);
         (*pSubChunksMap)[CHUNK_ID_LIST] = pNewListChunk;
+        NewChunkSize += LIST_HEADER_SIZE;
+        pFile->LogAsResized(this);
         return pNewListChunk;
     }
 
@@ -1223,6 +1244,7 @@ namespace RIFF {
       std::cout << "List::Readheader(ulong) ";
       #endif // DEBUG
         Chunk::ReadHeader(fPos);
+        if (CurrentChunkSize < 4) return;
         NewChunkSize = CurrentChunkSize -= 4;
         #if POSIX
         lseek(pFile->hFileRead, fPos + CHUNK_HEADER_SIZE, SEEK_SET);
@@ -1362,6 +1384,10 @@ namespace RIFF {
 // *************** File ***************
 // *
 
+//HACK: to avoid breaking DLL compatibility to older versions of libgig we roll the new std::set<Chunk*> into the old std::list<Chunk*> container, should be replaced on member variable level soon though
+#define _GET_RESIZED_CHUNKS() \
+	(reinterpret_cast<std::set<Chunk*>*>(ResizedChunks.front()))
+
     /** @brief Create new RIFF file.
      *
      * Use this constructor if you want to create a new RIFF file completely
@@ -1377,6 +1403,8 @@ namespace RIFF {
      * @see AddSubChunk(), AddSubList(), SetByteOrder()
      */
     File::File(uint32_t FileType) : List(this) {
+        //HACK: see _GET_RESIZED_CHUNKS() comment
+        ResizedChunks.push_back(reinterpret_cast<Chunk*>(new std::set<Chunk*>));
         #if defined(WIN32)
         hFileRead = hFileWrite = INVALID_HANDLE_VALUE;
         #else
@@ -1401,6 +1429,8 @@ namespace RIFF {
       std::cout << "File::File("<<path<<")" << std::endl;
       #endif // DEBUG
         bEndianNative = true;
+        //HACK: see _GET_RESIZED_CHUNKS() comment
+        ResizedChunks.push_back(reinterpret_cast<Chunk*>(new std::set<Chunk*>));
         #if POSIX
         hFileRead = hFileWrite = open(path.c_str(), O_RDONLY | O_NONBLOCK);
         if (hFileRead <= 0) {
@@ -1426,7 +1456,7 @@ namespace RIFF {
         Mode = stream_mode_read;
         ulStartPos = RIFF_HEADER_SIZE;
         ReadHeader(0);
-        if (ChunkID != CHUNK_ID_RIFF) {
+        if (ChunkID != CHUNK_ID_RIFF && ChunkID != CHUNK_ID_RIFX) {
             throw RIFF::Exception("Not a RIFF file");
         }
     }
@@ -1586,14 +1616,14 @@ namespace RIFF {
 
         // first we sum up all positive chunk size changes (and skip all negative ones)
         unsigned long ulPositiveSizeDiff = 0;
-        for (ChunkList::iterator iter = ResizedChunks.begin(), end = ResizedChunks.end(); iter != end; ++iter) {
+        std::set<Chunk*>* resizedChunks = _GET_RESIZED_CHUNKS();
+        for (std::set<Chunk*>::const_iterator iter = resizedChunks->begin(), end = resizedChunks->end(); iter != end; ++iter) {
             if ((*iter)->GetNewSize() == 0) {
                 throw Exception("There is at least one empty chunk (zero size): " + __resolveChunkPath(*iter));
             }
-            if ((*iter)->GetNewSize() + 1L > (*iter)->GetSize()) {
-                unsigned long ulDiff = (*iter)->GetNewSize() - (*iter)->GetSize() + 1L; // +1 in case we have to add a pad byte
-                ulPositiveSizeDiff += ulDiff;
-            }
+            unsigned long newSizePadded = (*iter)->GetNewSize() + (*iter)->GetNewSize() % 2;
+            unsigned long oldSizePadded = (*iter)->GetSize() + (*iter)->GetSize() % 2;
+            if (newSizePadded > oldSizePadded) ulPositiveSizeDiff += newSizePadded - oldSizePadded;
         }
 
         unsigned long ulWorkingFileSize = GetFileSize();
@@ -1611,9 +1641,9 @@ namespace RIFF {
             #else
             int iBytesMoved = 1;
             #endif
-            for (unsigned long ulPos = 0; iBytesMoved > 0; ulPos += iBytesMoved) {
-                const unsigned long ulToMove = ulFileSize - ulPos;
-                iBytesMoved = (ulToMove < 4096) ? ulToMove : 4096;
+            for (unsigned long ulPos = ulFileSize; iBytesMoved > 0; ) {
+                iBytesMoved = (ulPos < 4096) ? ulPos : 4096;
+                ulPos -= iBytesMoved;
                 #if POSIX
                 lseek(hFileRead, ulPos, SEEK_SET);
                 iBytesMoved = read(hFileRead, pCopyBuffer, iBytesMoved);
@@ -1643,7 +1673,7 @@ namespace RIFF {
         if (ulTotalSize < ulActualSize) ResizeFile(ulTotalSize);
 
         // forget all resized chunks
-        ResizedChunks.clear();
+        resizedChunks->clear();
     }
 
     /** @brief Save changes to another file.
@@ -1700,7 +1730,7 @@ namespace RIFF {
         if (ulTotalSize < ulActualSize) ResizeFile(ulTotalSize);
 
         // forget all resized chunks
-        ResizedChunks.clear();
+        _GET_RESIZED_CHUNKS()->clear();
 
         #if POSIX
         if (hFileWrite) close(hFileWrite);
@@ -1743,14 +1773,18 @@ namespace RIFF {
         #else
         if (hFileRead) fclose(hFileRead);
         #endif // POSIX
+        DeleteChunkList();
+        pFile = NULL;
+        //HACK: see _GET_RESIZED_CHUNKS() comment
+        delete _GET_RESIZED_CHUNKS();
     }
 
     void File::LogAsResized(Chunk* pResizedChunk) {
-        ResizedChunks.push_back(pResizedChunk);
+        _GET_RESIZED_CHUNKS()->insert(pResizedChunk);
     }
 
     void File::UnlogResized(Chunk* pResizedChunk) {
-        ResizedChunks.remove(pResizedChunk);
+        _GET_RESIZED_CHUNKS()->erase(pResizedChunk);
     }
 
     unsigned long File::GetFileSize() {
