@@ -2,7 +2,7 @@
  *                                                                         *
  *   libgig - C++ cross-platform Gigasampler format file access library    *
  *                                                                         *
- *   Copyright (C) 2003-2007 by Christian Schoenebeck                      *
+ *   Copyright (C) 2003-2009 by Christian Schoenebeck                      *
  *                              <cuse@users.sourceforge.net>               *
  *                                                                         *
  *   This library is free software; you can redistribute it and/or modify  *
@@ -52,7 +52,19 @@
 # include <unistd.h>
 #endif // POSIX
 
+#ifdef _MSC_VER
+// Visual C++ 2008 doesn't have stdint.h
+typedef __int8 int8_t;
+typedef __int16 int16_t;
+typedef __int32 int32_t;
+typedef __int64 int64_t;
+typedef unsigned __int8 uint8_t;
+typedef unsigned __int16 uint16_t;
+typedef unsigned __int32 uint32_t;
+typedef unsigned __int64 uint64_t;
+#else
 #include <stdint.h>
+#endif
 
 #ifdef WIN32
 # include <windows.h>
@@ -142,12 +154,12 @@ namespace RIFF {
         public:
             Chunk(File* pFile, unsigned long StartPos, List* Parent);
             String         GetChunkIDString();
-            uint32_t       GetChunkID() { return ChunkID; };            ///< Chunk ID in unsigned integer representation.
-            List*          GetParent()  { return pParent; };            ///< Returns pointer to the chunk's parent list chunk.
-            unsigned long  GetSize()    { return CurrentChunkSize; };   ///< Chunk size in bytes (without header, thus the chunk data body)
-            unsigned long  GetNewSize() { return NewChunkSize;     };   ///< New chunk size if it was modified with Resize().
-            unsigned long  GetPos()     { return ulPos; };              ///< Position within the chunk data body
-            unsigned long  GetFilePos() { return ulStartPos + ulPos; }; ///< Current, actual offset in file.
+            uint32_t       GetChunkID() { return ChunkID; }             ///< Chunk ID in unsigned integer representation.
+            List*          GetParent()  { return pParent; }             ///< Returns pointer to the chunk's parent list chunk.
+            unsigned long  GetSize()    { return CurrentChunkSize; }    ///< Chunk size in bytes (without header, thus the chunk data body)
+            unsigned long  GetNewSize() { return NewChunkSize;     }    ///< New chunk size if it was modified with Resize().
+            unsigned long  GetPos()     { return ulPos; }               ///< Position within the chunk data body
+            unsigned long  GetFilePos() { return ulStartPos + ulPos; }  ///< Current, actual offset in file.
             unsigned long  SetPos(unsigned long Where, stream_whence_t Whence = stream_start);
             unsigned long  RemainingBytes();
             stream_state_t GetState();
@@ -271,6 +283,7 @@ namespace RIFF {
             void LoadSubChunksRecursively();
             virtual unsigned long WriteChunk(unsigned long ulWritePos, unsigned long ulCurrentDataOffset);
             virtual void __resetPos(); ///< Sets List Chunk's read/write position to zero and causes all sub chunks to do the same.
+            void DeleteChunkList();
     };
 
     /** @brief RIFF File
@@ -310,7 +323,7 @@ namespace RIFF {
             friend class List;
         private:
             stream_mode_t  Mode;
-            ChunkList      ResizedChunks; ///< All chunks which have been resized (enlarged / shortened).
+            ChunkList ResizedChunks; ///< All chunks which have been resized (enlarged / shortened).
 
             unsigned long GetFileSize();
             void ResizeFile(unsigned long ulNewSize);
@@ -330,9 +343,9 @@ namespace RIFF {
         public:
             String Message;
 
-            Exception(String Message) { Exception::Message = Message; };
+            Exception(String Message) { Exception::Message = Message; }
             void PrintMessage();
-            virtual ~Exception() {};
+            virtual ~Exception() {}
     };
 
     String libraryName();
